#!/usr/bin/env python
# Copyright (C) 2007-2014 Canonical, Ltd.
# Author: Kees Cook <kees@ubuntu.com>
#         Jamie Strandboge <jamie@canonical.com>
# License: GPLv3
import os, sys, tempfile, time, shutil, launchpadlib
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import Credentials
from launchpadlib.errors import HTTPError
import urllib, urllib2, cookielib
try:
    import progressbar
except:
    pass

version_min = [1,5,7]
if [int(x) for x in launchpadlib.__version__.split('.')] < version_min:
    raise ValueError, "Requires launchpadlib version %s or later (%s in use)" \
        % (".".join(version_min), launchpadlib.__version__)

def connect(use_edge=False, beta=False, version=None, uri=None, bot=None):

    # Use of edge is obsoleted
    # See: http://blog.launchpad.net/general/edge-is-deprecated
    if use_edge == True:
        raise ValueError, "Use of use_edge=True with connect() is obsoleted, see comments in lpl_common.py."

    cachedir = os.path.expanduser('~/.launchpadlib/cache')
    if not os.path.exists(cachedir):
        os.makedirs(cachedir,0700)

    if beta:
        version="beta"
    elif not version:
        version="1.0"
    elif version:
        version=version

    root = 'production'
    credfile = os.path.expanduser('~/.launchpadlib/credentials-lpnet')
    if uri:
        root = uri
        credfile = os.path.expanduser('~/.launchpadlib/credentials-%s' % (uri.replace('/','_')))
    if bot:
        credfile = credfile.replace('credentials', 'bot-credentials')
    launchpad = Launchpad.login_with(sys.argv[0], service_root=root, launchpadlib_dir=cachedir, credentials_file=credfile, version=version)
    return launchpad

def save(item):
    # attempt to deal with intermittent failures
    count = 0
    max_tries = 10
    result = False
    err_str = ""
    while not result and count < max_tries:
        try:
            item.lp_save()
            result = True
        except HTTPError, error:
            err_str = "save() failed: %s" % (error.content)
            count += 1
            time.sleep(5)

    if not result:
        print >>sys.stderr, "%s (tried %d times)" % (err_str, max_tries)
    return result

def extract_task(task):
    name = task.bug_target_name.lower()
    package = None
    distribution = None
    series = None
    if ' (' in name:
        package, distribution = name.split(' (',1)
        distribution = distribution.split(')')[0]
    else:
        package = name
    if distribution and ' ' in distribution:
        distribution, series = distribution.split(' ',1)

    return package, distribution, series

def split_ppa(group):
    if '/' in group:
        group, ppa = group.split('/',1)
    else:
        ppa = 'ppa'
    return group, ppa

def get_archive(name, lp, verbose=False, distribution=None):
    if name == 'ubuntu':
        if verbose:
            print "Loading Ubuntu Archive ..."
        if distribution == None:
            distribution = lp.distributions[name]
        archive, partner = distribution.archives
        group = 'ubuntu'
        ppa = None
    else:
        group, ppa = split_ppa(name)
        if verbose:
            print "Loading %s '%s' PPA ..." % (group, ppa)
        if distribution is None:
            archive = lp.people[group].getPPAByName(name=ppa)
        else:
            archive = lp.people[group].getPPAByName(name=ppa,
                                                    distribution=distribution)
    return archive, group, ppa


def opener_with_cookie(cookie_file):
    import sqlite3 as sqlite

    # Work around Firefox 3.5's dumb sqlite locking problems by copying cookies out:
    tmp = None
    if cookie_file.endswith('.sqlite'):
        (cookie_path, cookie_name) = os.path.split(cookie_file)
        sql = tempfile.NamedTemporaryFile(prefix='cookies-XXXXXX', suffix='.sqlite')
        shutil.copy(cookie_file, sql.name)

        match = '%launchpad.net'
        con = sqlite.connect(sql.name)
        cur = con.cursor()
        cur.execute("select host, path, isSecure, expiry, name, value from moz_cookies where host like ?", [match])
        ftstr = ["FALSE","TRUE"]
        tmp = tempfile.NamedTemporaryFile(prefix='cookies-XXXXXX', suffix='.mozilla')
        cookie_file = tmp.name
        tmp.write("# HTTP Cookie File\n")
        for item in cur.fetchall():
            str = "%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % ( item[0], \
                   ftstr[item[0].startswith('.')], item[1], \
                   ftstr[item[2]], item[3], item[4], item[5])
            tmp.write(str)
        sql = None
        tmp.flush()

    cj = cookielib.MozillaCookieJar()
    cj.load(cookie_file)
    opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
    tmp = None
    return opener

def open_url(opener, url):
    tries = 0
    max_tries = 10
    while True:
        try:
            page = opener.open(url)
            break
        except urllib2.HTTPError, e:
            tries += 1
            if tries >= max_tries or (e.code not in (500, 502, 503, 504)):
                print >>sys.stderr, "Failed (%s): %d %s" % (url, e.code, e.msg)
                raise
            time.sleep(3)

    return page

def chunked_read(response, chunk_size=8192, outfile=None, verbose=True):
    total_size = int(response.info().getheader('Content-Length').strip())
    bytes_so_far = 0

    try:
        widgets = [progressbar.Percentage(), ' ',
                   progressbar.Bar(marker='=', left='[', right=']'),
                   ' ', str(total_size), ' ', progressbar.ETA()]
    except:
        raise ValueError, "Please install python-progressbar to use 'download'"

    bar = None
    if verbose and sys.stdout.isatty():
        bar = progressbar.ProgressBar(widgets=widgets, maxval=total_size).start()

    while 1:
        chunk = response.read(chunk_size)
        if not chunk:
            break
        bytes_so_far += len(chunk)
        if bar:
            bar.update(bytes_so_far)
        if outfile:
            outfile.write(chunk)

    if bar:
        bar.finish()
    return bytes_so_far


def download(opener, url, filename=None, verbose=True, dryrun=False):
    if verbose:
        print "  %s ..." % (url)
    if dryrun:
        print "(dry run, skipping)"
        return

    if not filename:
        filename = os.path.basename(urllib.unquote(url))

    if os.path.exists(filename):
        print "(already exists, skipping)"
        return

    tmp = tempfile.NamedTemporaryFile(delete=False)

    response = open_url(opener, url)
    chunked_read(response, outfile=tmp, verbose=verbose)
    tmp.close()
    shutil.move(tmp.name,filename)

