#!/usr/bin/env python
#
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2010 Canonical, Ltd.
# License: GPLv3
#
# Display the quantity of types of actions taken by people or team members
# on public Ubuntu bug reports.

import lpl_common, optparse

from mailbox import PortableUnixMailbox
from email import message_from_file
from email.errors import MessageParseError
from email.utils import parseaddr, parsedate, getaddresses
from sys import argv, stderr
from operator import itemgetter
from datetime import datetime, date


MESSAGE_NOT_PARSEABLE = object()

actions = { 'retitled': 
                {'body':'Summary changed',
                 'legend':'Titles modified'},
            'description':
                {'body':'Description changed',
                 'legend':'Descriptions modified'},
            'pkgassigned':
                {'body':'Sourcepackagename: None',
                 'legend':'Assigned to package'},
            'tagged':
                {'body':'Tags added',
                 'legend':'Added tags'},
            'watched':
                {'body':'Bug watch added',
                 'legend':'Added bug watches'},
            'duped':
                {'body':'This bug is a duplicate of',
                 'legend':'Marked as duplicate'},
            'unduped':
                {'body':'This bug is no longer a duplicate of',
                 'legend':'Unmarked as duplicate'},
            'incompleted':
                {'body':'=> Incomplete',
                 'legend':'Incompleted'},
            'confirmed':
                {'body':'=> Confirmed',
                 'legend':'Confirmed'},
            'triaged':
                {'body':'=> Triaged',
                 'legend':'Triaged'},
            'inprogressed':
                {'body':'=> In Progress',
                 'legend':'In Progessed'},
            'fixcommitted':
                {'body':'=> Fix Committed',
                 'legend':'Fix Committed'},
            'fixreleased':
                {'body':'=> Fix Released',
                 'legend':'Fix Released'},
            'wontfixed':
                {'body':'''=> Won't Fix''',
                 'legend':'''Won't Fixed'''},
            'invalidated':
                {'body':'=> Invalid',
                 'legend':'Invalidated'},
            'milestoned':
                {'body':'Target: ',
                 'legend':'Set a milestone'},
            'converted':
                {'body':'bug changed to question:',
                 'legend':'Converted to a question'},
            'patched':
                {'body':'** Patch added: ',
                 'legend':'Added a patch'},
          }


def find_team_members(team_name):

    launchpad = lpl_common.connect()
    team = launchpad.people[team_name]
    team_members = [member for member in team.members]
    team_member_names = []

    if not opt.recurse:
        team_member_names = [member.name for member in team_members]
    elif opt.recurse:
        for team_member in team_members:
            if team_member.is_team:
                team_member_names.extend([member.name for member in
                                         team_member.members])
            else:
                team_member_names.append(team_member.name)

    return team_member_names


def message_factory(fp):
    try:
        return message_from_file(fp)
    except MessageParseError:
        # Don't return None since that will stop the mailbox iterator.
        return MESSAGE_NOT_PARSEABLE


def show_progress(iterator, interval=100):
    """Show signs of progress."""
    for count, item in enumerate(iterator):
        if count % interval == 0:
            stderr.write('.')
        yield item


def scan_bugs(messages):
    for count, message in enumerate(messages):
        # Skip broken messages.
        if message is MESSAGE_NOT_PARSEABLE:
            continue

        # Skip replies.
        # Every message has as a reference in it for some reason
#        references = message['references']
#        if references is not None:
#            yield 'reply'
#            continue

        # Check it's from a Launchpad bug.
        reply_to = message['reply-to']
        if reply_to is None:
            continue
        reply_name, reply_address = parseaddr(reply_to)
        reply_local_part, reply_domain = reply_address.split('@')
        if not (reply_domain == 'bugs.launchpad.net' and
                reply_local_part.isdigit()):
            continue
        bug_id = int(reply_local_part)

        # Only new bugs.
        subject = message.get('subject', '')
        xlpbt = message.get('X-Launchpad-Bug-Tags', '')
        #if 'precise' not in xlpbt:
        #    continue
        if not '[NEW]' in subject:
            # X-Launchpad-Bug-Modifier is only available recently
            xlpbm = message.get('X-Launchpad-Bug-Modifier', '')
            if xlpbm != '':
                sender_name = xlpbm.split(' ')[-1].strip('()')
            else:
                sender = message.get('from', '')
                sender_name, sender_address = parseaddr(sender)
            mail_date_tuple = parsedate(message['date'])
            mail_date = datetime(mail_date_tuple[0], mail_date_tuple[1],
                                 mail_date_tuple[2]).date()
            if sender_name not in team_members:
                continue
            if mail_date < begin or mail_date > end:
                continue
            payload = message.get_payload()

            if opt.action != "all":
                if actions[opt.action]['body'] in payload:
                    yield '%s' % (sender_name)
            else:
                for action in actions.keys():
                    if actions[action]['body'] in payload:
                        yield '%s %s' % (sender_name, actions[action]['legend'])

if __name__ == '__main__':

    parser = optparse.OptionParser()
    parser.set_defaults(action="all", recurse=False, csv=False)
    parser.add_option("--team", help="Launchpad team name",
                      metavar="TEAM_NAME")
    parser.add_option("--all-members", help="Use members that are members of \
                                             a subteam.", dest="recurse",
                      action="store_true")
    parser.add_option("--person", help="Launchpad user",
                      metavar="PERSON")
    parser.add_option("--mbox", help="Bugs mailing list mbox(es)",
                      metavar="MBOX,MBOX,...")
    parser.add_option("--progress", help="Show activity indicator")
    parser.add_option("--action", help="Action type to search for - one of \
                                        retitled, description, pkgassigned, \
                                        tagged, watched, duped, unduped, \
                                        incompleted, confirmed, triaged, \
                                        inprogressed, fixcommitted, \
                                        fixreleased, wontfixed, invalidated, \
                                        milestoned, converted or all",
                      dest="action")
    parser.add_option("--since", help="Starting date %Y-%m-%d")
    parser.add_option("--before", help="Ending date %Y-%m-%d")
    parser.add_option("--csv", help="Output data to a csv file", action='store_true')

    (opt, args) = parser.parse_args()
    counts = {}

    if opt.person:
        team_members = []
        team_members.append(opt.person)

    if opt.team:
        team_members = find_team_members(opt.team)

    if opt.since:
        begin = datetime.strptime(opt.since, "%Y-%m-%d").date()
    else:
        begin = date(2001, 01, 01)

    if opt.before:
        end = datetime.strptime(opt.before, "%Y-%m-%d").date()
    else:
        end = date.today()

    mbox_list = opt.mbox.split(',')

    for mbox in mbox_list:
        mailbox = PortableUnixMailbox(open(mbox, 'rb'), message_factory)

        if opt.progress:
            for test in scan_bugs(show_progress(mailbox)):
                counts[test] = counts.setdefault(test, 0) + 1
        else:
            for test in scan_bugs(mailbox):
                counts[test] = counts.setdefault(test, 0) + 1

    if opt.csv is True and opt.team:
        # write out data to a csv file named after the action
        datafile = open('%s-%s.csv' % (opt.team, opt.action), 'w')
        for k, v in sorted(counts.items(), key=itemgetter(1), reverse=True):
            datafile.write('%s, %d\n' % (k, v))
        datafile.close()
    else:
        for k, v in sorted(counts.items(), key=itemgetter(1), reverse=True):
            print '%s: %d' % (k, v)
