#!/usr/bin/env python
#
# This script looks up members of a team in Launchpad and then uses a -changes
# mbox file to determine what packages were uploaded by team members.  It
# additionally returns the number of times each package was uploaded.
#
# Example usage:
# ml-package-uploads-by-person.py --team ubuntu-desktop -m jaunty-changes.mbox
#
# Copyright 2011 Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# Licensed under the GNU General Public License, version 3.

from mailbox import PortableUnixMailbox
from email import message_from_file
from email.errors import MessageParseError
from sys import stderr, exit
from operator import itemgetter
from optparse import OptionParser
from datetime import *

# launchpadlib bits
from launchpadlib.errors import HTTPError

import lpl_common

MESSAGE_NOT_PARSEABLE = object()


def message_factory(fp):
    try:
        return message_from_file(fp)
    except MessageParseError:
        # Don't return None since that will stop the mailbox iterator.
        return MESSAGE_NOT_PARSEABLE


def show_progress(iterator, interval=100):
    """Show signs of progress."""
    for count, item in enumerate(iterator):
        if count % interval == 0:
            stderr.write('.')
        yield item


def find_team_emails(team_name):
    try:
        team = launchpad.people[team_name]
    except KeyError:
        print >> stderr.write("No exact match for team: %s") % team_name
        exit(1)
    team_emails = []
    for member in team.participants:
        try:
            #if member.hide_email_addresses is False:
            for email in member.confirmed_email_addresses:
                team_emails.append(email.email)
            if member.preferred_email_address:
                team_emails.append(member.preferred_email_address.email)
        except HTTPError, error:
            # skip over deactivated accounts
            if error.response['status'] == '410':
                continue
            else:
                print stderr.write("Error '%s' with member '%s'") % (error, member)
    return team_emails


def package_in_dev_release(dev_release, package_name):
    if dev_release.getSourcePackage(name=package_name):
        package = dev_release.getSourcePackage(name=package_name)
        component = package.latest_published_component_name
        return component
    else:
        return None


def scan_bugs(messages, include=None):
    fixer = ""
    for count, message in enumerate(messages):
        # Skip broken messages.
        if message is MESSAGE_NOT_PARSEABLE:
            continue

        for part in message.walk():
            if not part.is_multipart():
                lines = part.get_payload(decode=True).splitlines()
                sourcepackage = None
                for line in lines:
                    if line.startswith('Date: '):
                        message_date = line[6:]
                    # use message_date for debugging
                    if line.startswith('Source: '):
                        sourcepackage = line[8:].split()[0]
                    if line.startswith('Distribution: '):
                        distribution = line[14:].split()[0]
                    if line.startswith('Changed-By: '):
                        fixer = line[12:].split()[-1].strip('<>')
                        if fixer is None:
                            print "message_date: %s has no fixer" % message_date
                            break
                        elif include is not None and fixer not in include:
                            #print "skipping not included fixer: %s" % fixer
                            break #continue
                        else:
                            continue
                if sourcepackage is not None:
                    if sourcepackage not in packages.keys():
                        packages[sourcepackage] = {}
                    if not include:
                        packages[sourcepackage][fixer] = packages[sourcepackage].setdefault(fixer, 0) + 1
                    elif fixer in include:
                        packages[sourcepackage][fixer] = packages[sourcepackage].setdefault(fixer, 0) + 1

if __name__ == '__main__':

    parser = OptionParser()
    parser.add_option("-m", "--mbox", metavar="FILE",
        help="a changes mailing list mbox file to scan")
    parser.add_option("-t", "--team", default=None,
        help="display fixes only from this Launchpad team name")
    (opt, args) = parser.parse_args()

    launchpad = lpl_common.connect()
    ubuntu = launchpad.projects['ubuntu']
    dev_release = ubuntu.current_series
    packages = {}

    if opt.mbox:
        mbox_list = opt.mbox.split(',')
        for mbox in mbox_list:
            mailbox_file = mbox
            mailbox = PortableUnixMailbox(
                open(mailbox_file, 'rb'), message_factory)

            release = opt.mbox.split('-')[0]
            if opt.team:
                team_name = opt.team
                team_members = find_team_emails(team_name)
                scan_bugs(mailbox, include=team_members)
            else:
                scan_bugs(mailbox)

    else:
        print >> stderr.write("An mbox file is required for finding uploads.")
        exit(1)

    for package_name in sorted(packages.keys()):
        # check to make sure the package is in the development release
        if package_in_dev_release(dev_release, package_name) is None:
            continue
        else:
            component = package_in_dev_release(dev_release, package_name)
            for k, v in sorted(packages[package_name].items(),
                key=itemgetter(1), reverse=True):
                print "%s, %s, %s, %s" % (package_name, component, k, v)
