#!/usr/bin/python
#
# Return the package names and quantity of bug reported over the last X number
# of days for the top 5 packages receiving the most bug reports
#
# Copyright 2011  Canonical, Ltd.
# Author: Brian Murray <brian@canonical.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
# the full text of the license.

from mailbox import PortableUnixMailbox
from email import message_from_file
from email.errors import MessageParseError
from email.utils import parseaddr, parsedate
from operator import itemgetter
from sys import stderr, exit
from collections import deque
import datetime
import optparse


MESSAGE_NOT_PARSEABLE = object()

# from http://www.daniweb.com/forums/thread42429.html#
class RingBuffer(deque):

    def __init__(self, size):
        deque.__init__(self)
        self.size = size

    def full_append(self, item):
        deque.append(self, item)
        # full, pop the oldest item which is the left most item
        self.popleft()

    def append(self, item):
        deque.append(self, item)
        # max size reached, append becomes full_append
        if len(self) == self.size:
            self.append = self.full_append

    def get(self):
        """returns a list of size items (newest items)"""
        return list(self)

headers = { 'distribution=ubuntu; sourcepackage=None':'no package bugs',
            'distribution=ubuntu.*component=main':'main repo bugs',
            'distribution=ubuntu.*component=universe':'universe repo bugs',
            'distribution=ubuntu.*component=multiverse':'multiverse repo bugs',
            'distribution=ubuntu.*component=restricted':'restricted repo bugs'
          }

def message_factory(fp):
    try:
        return message_from_file(fp)
    except MessageParseError:
        # Don't return None since that will stop the mailbox iterator.
        return MESSAGE_NOT_PARSEABLE


def show_progress(iterator, interval=100):
    """Show signs of progress."""
    for count, item in enumerate(iterator):
        if count % interval == 0:
            stderr.write('.')
        yield item


def scan_bugs(messages, timeframe):
    for count, message in enumerate(messages):
        # Skip broken messages.
        if message is MESSAGE_NOT_PARSEABLE:
            continue

        # Skip replies.
        # Every message has as a reference in it for some reason
#        references = message['references']
#        if references is not None:
#            yield 'reply'
#            continue

        # Check it's from a Launchpad bug.
        reply_to = message['reply-to']
        if reply_to is None:
            continue
        reply_name, reply_address = parseaddr(reply_to)
        reply_local_part, reply_domain = reply_address.split('@')
        if not (reply_domain == 'bugs.launchpad.net' and
                reply_local_part.isdigit()):
            continue
        bug_id = int(reply_local_part)

        # Only new bugs.
        subject = message.get('subject', '')
        if '[NEW]' in subject and not 'Re:' in subject:
            #yield 'NEW bug reports'
            x_lp_bug = message.get('x-launchpad-bug', '')
            sent = parsedate(message['date'])

            if (datetime.datetime(sent[0], sent[1], sent[2]) <
                (datetime.datetime.today() - datetime.timedelta(timeframe))):
                continue
            for string in headers.keys():
                #if search(r'%s' % (string), x_lp_bug ):
                # only return the packages in main for now
                # easy to change it for all packages by removing the main component
                # FAIL for packages that have changed from universe to main they still show up in universe
                # LP: #297709
                if 'sourcepackage' in x_lp_bug:
                    sourcepackage = x_lp_bug.split(';')[1].split('=')[1]
                    yield '%s' % sourcepackage
                    # is this only returning the first bug task?
                    break

if __name__ == '__main__':

    parser = optparse.OptionParser()
    parser.add_option("--mbox", help="Bugs mailing list mbox(es)",
        metavar="MBOX,MBOX,...")
    parser.add_option("--days",
        help="Review last number of days of bug reports")

    (opt, args) = parser.parse_args()
    mbox_list = opt.mbox.split(',')

    if opt.days.isdigit():
        timeframe = int(opt.days)
    elif opt.days.isdigit() == False:
        print 'An integer is required for the number of days'
        exit(1)
    else:
        timeframe = 1

    for mailbox_file in mbox_list:
        mailbox = PortableUnixMailbox(
            open(mailbox_file, 'rb'), message_factory)

        size = 1000
        ring = RingBuffer(size)
        counts = {}
        for test in scan_bugs(mailbox, timeframe):
            ring.append(test)

        packages = ring.get()

        for package in packages:
            counts[package] = counts.setdefault(package, 0) + 1

        total = 0
        number_of_packages = 0

        print "Top 5 Packages with New bugs in the past %s days" % (opt.days)

        for pkg, count in sorted(counts.items(), key=itemgetter(1),
            reverse=True):

            print '  %s: %d' % (pkg, count)
            total += count
            number_of_packages += 1
            if number_of_packages == 5:
                break
